#!/bin/bash
#Description:Cooperation with trace_net_flow and trace_net_flow_kern.o.* built in ebpf_build_prog folder.
#            Used to find matched trace_net_flow_kern.o file according os_type and kernel mainline version,
#            run trace_net_flow exe finally.
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2021. All rights reserved.
#History: 2021-11-07 created

current_dir=$(cd $(dirname "$0"); pwd)
cd ${current_dir}
DEBUGFS_PATH="/sys/kernel/debug"

function usage() {
    if [[ $# -ne 2 ]]; then
        echo "Use method:./trace_net_flow [interval(seconds)] [count]"
        exit 1
    fi
}

function kernel_version_check() {
    uname_a=$(cat /etc/os-release |grep ^PRETTY_NAME= |grep -Eo "\".+\"")
    echo "uname_a=${uname_a}"
    version_id=$(cat /usr/include/linux/version.h |grep "#define LINUX_VERSION_CODE" |grep -Eo "[0-9]+")
    echo "version_id=${version_id}"

    case ${uname_a} in
        # Debian系列操作系统/proc/sys/kernel/version中保存了内核mainline 版本信息
        *"Debian"*)
            kernel_version_id=$(cat /proc/sys/kernel/version |grep -Eo "[0-9]+\.[0-9]+\.[0-9]+"|awk -F. '{print $1*65536+$2*256+$3}')
            ;;
        *"\"Linx GNU"*)
            kernel_version_id=$(cat /proc/sys/kernel/version |grep -Eo "[0-9]+\.[0-9]+\.[0-9]+"|awk -F. '{print $1*65536+$2*256+$3}')
            ;;
        # Ubuntu系列操作系统只能从/usr/include/linux/version.h获取内核mainline 版本信息
        "\"Ubuntu "*)
            kernel_version_id=${version_id}
            ;;
        "\"uos "*)
            kernel_version_id=${version_id}
            ;;
        # 其它操作系统从uname -r获取内核mainline 版本信息
        *)
            kernel_version_id=$(uname -r |grep -Eo "^[0-9]+\.[0-9]+\.[0-9]+"|awk -F. '{print $1*65536+$2*256+$3}')
            ;;
    esac

    echo "kernel_version_id=${kernel_version_id}"

    # 如果/proc/sys/kernel/version信息存在，优先使用
    if [[ -z ${kernel_version_id} ]]; then
        kernel_version_id=$(cat /proc/sys/kernel/version |grep -Eo "[0-9]+\.[0-9]+\.[0-9]+"|awk -F. '{print $1*65536+$2*256+$3}')
    fi
    # 只要用户使用了与内核版本匹配的user space  development header，/usr/include/linux/version.h是正确的。
    if [[ -z ${kernel_version_id} ]]; then
        kernel_version_id=${version_id}
    fi
    # readhat系列操作系统，uname -r显示的信息与内核mainline是一致的
    if [[ -z ${kernel_version_id} ]]; then
        kernel_version_id=$(uname -r |grep -Eo "^[0-9]+\.[0-9]+\.[0-9]+"|awk -F. '{print $1*65536+$2*256+$3}')
    fi

    if [[ -z ${kernel_version_id} ]]; then
        echo "Get kernel_version_id failed!"
        exit 2
    fi

    if [[ -f "./trace_net_flow_kern.o.${kernel_version_id}" ]]; then
        cp -f "trace_net_flow_kern.o.${kernel_version_id}" trace_net_flow_kern.o
    else
        echo "No matched version of trace_net_flow_kern.o"
        exit 3
    fi
}

function main() {
    interval=$1
    count=$2
    debugfs_flag=""
    usage $@
    kernel_version_check
    debugfs_flag=$(mount -l | grep "type debugfs")
    if [[ -z "${debugfs_flag}" ]]; then
        mount -t debugfs debugfs "${DEBUGFS_PATH}"
        mount_result=$?
        if [[ "${mount_result}" -ne "0" ]]; then
            echo "Mount debugfs failed"
            exit 4
        fi
    fi
    ./trace_net_flow ${interval} ${count}
    trace_result=$?
    if [[ -z "${debugfs_flag}" ]]; then
        umount "${DEBUGFS_PATH}" >> /dev/null 2>& 1
    fi
    exit "${trace_result}"
}

main $@
