#!/bin/bash
#Description: plugin agent install entry
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2023. All rights reserved.
#History: 2023-6-17 created
current_dir=$(cd $(dirname "$0"); pwd)
source "${current_dir}/utils.sh"
install_path="$1/DevKit/"
INSTALL_LOG="${install_path}/devkitplugins/sys_diagnosis/deploy/sys_diagnosis_agent_install.log"
touch "${INSTALL_LOG}"
chmod 600 "${INSTALL_LOG}"


EXIT_FLAG="TRUE" # 记录exit_clean状态防止进入死循环
trap "agent_exit_clean" EXIT

# 检查是否有安装残留并清理
function clean_residue() {
    echo "Clean sys_diagnosis residue"
    delete_sudo_tools "${INSTALL_LOG}"
    local sys_diagnosis_install_path="${install_path}/devkitplugins/sys_diagnosis/"
    if [[ -d "${sys_diagnosis_install_path}" ]]; then
        rm -rf "${sys_diagnosis_install_path}"
    fi
    echo "Clean sys_diagnosis residue success"
}

function agent_exit_clean() {
    local exit_code="$?"
    if [[ "${exit_code}" -ne "${SUCCESS}" ]] && [[ ${EXIT_FLAG} == "TRUE" ]]; then
        EXIT_FLAG="FALSE"
        clean_residue
        exit "${ERROR}"
    fi
}

function installed_check()
{
    if [[ -f "${SUDO_FILE}" ]]; then
        echo_and_log "Sys_diagnosis agent has already been deployed!" "${ERROR}" "${INSTALL_LOG}"
        exit "${ERROR}"
    fi
}

function init_variable() {
    if [[ ! -f "${CONFIG_FILE}" ]]; then
        echo_and_log "There is no configure file(${CONFIG_FILE}), please check!" "${ERROR}"
        exit "${ERROR}"
    fi
    INSTALL_PLUGIN_PATH="${install_path}/devkitplugins/"
}

function agent_libcheck()
{
    echo_and_log "\nCheck agent Dependent Packages" "${DOING}" "${INSTALL_LOG}"
    bash "${current_dir}/env_check.sh" "${install_path}" "agent"
    if [[ "$?" -ne "${SUCCESS}" ]]; then
        echo_and_log "Check install Dependent Packages Failed, Stop installing sys_diagnosis agent" "${ERROR}" "${INSTALL_LOG}"
        exit "${ERROR}"
    else
        echo_and_log "Check install Dependent Packages Success" "${SUCCESS}" "${INSTALL_LOG}"
    fi
}

function change_file_path_permission() {
    find "${INSTALL_PLUGIN_PATH}/sys_diagnosis/deploy" -name "*.sh" -exec chown root:root {} \; >> /dev/null 2>&1
    if [[ -d "${INSTALL_PLUGIN_PATH}/sys_diagnosis" ]]; then
        find "${INSTALL_PLUGIN_PATH}/sys_diagnosis" -type d -exec chmod 700 {} \; >> /dev/null 2>&1
        chmod 755 "${INSTALL_PLUGIN_PATH}/sys_diagnosis"
    fi
    local deploy_path="${INSTALL_PLUGIN_PATH}/sys_diagnosis/deploy/"
    if [[ -d "${deploy_path}" ]]; then
        chown root:root "${deploy_path}"
        chown root:root "${INSTALL_PLUGIN_PATH}/sys_diagnosis"
        chown root:root "${INSTALL_PLUGIN_PATH}"
    fi
    # 所有so权限更改为550
    find "${INSTALL_PLUGIN_PATH}/sys_diagnosis/" -type f -name "*.so" -print0 | xargs -0 chmod 550 >> /dev/null 2>&1
    find "${INSTALL_PLUGIN_PATH}/sys_diagnosis/" -type f -name "*.so.*" -print0 | xargs -0 chmod 550 >> /dev/null 2>&1
}

function create_log_dir() {
    local log_dir="${install_path}/logs/"
    if [[ ! -d "${log_dir}" ]]; then
        mkdir -p "${log_dir}"
        chown "${DEVKIT_USER}":"${DEVKIT_USER}" "${log_dir}"
        chmod 755 "${log_dir}"
    fi
}


# 删除无用的文件
function rm_useless_files() {
    local deploy_dir="${install_path}/devkitplugins/sys_diagnosis/deploy/"
    if [[ -d "${deploy_dir}" ]]; then
        local rm_files=("agent_install_entry.sh" "install.sh" "env_check.sh" "uninstall.sh")
        for file in ${rm_files[@]}
        do
            if [[ -f "${deploy_dir}/${file}" ]]; then
                rm -f "${deploy_dir}/${file}"
                echo_and_log "Remove ${file} success" "${SUCCESS}" "${INSTALL_LOG}"
            else
                echo_and_log "Remove ${file} failed, continue" "${DOING}" "${INSTALL_LOG}"
            fi
        done
    fi
}


function main() {
    init_variable
    check_kunpeng "${INSTALL_LOG}"
    installed_check
    echo_and_log "Installing sys_diagnosis agent" "${DOING}" "${INSTALL_LOG}"
    agent_libcheck
    copy_sudo_files "${install_path}" "${INSTALL_LOG}" "agent"
    add_sudoers "${install_path}" "${INSTALL_LOG}"
    install_third_app "${INSTALL_LOG}"
    create_log_dir
    change_file_path_permission
    echo_and_log "Installing sys_diagnosis agent success" "${SUCCESS}" "${INSTALL_LOG}"
    rm_useless_files
    # 所有安装完成后，更改日志文件权限为只读
    chmod 400 "${INSTALL_LOG}"
}


main $@
