#!/bin/bash
#Description: const params
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2020. All rights reserved.
#History: 2020-05-11 created

# 获取创建用户默认路径
function get_home_path() {
    # 默认目录为/home # 用户自定义的用户家目录 默认为home
    local useradd_file="/etc/default/useradd"
    if [ ! -f ${useradd_file} ]; then
        USER_HOME="/home"
        return
    fi
    # 当配置为空时，默认家目录为根目录
    if grep -E "^HOME=$" "${useradd_file}" >> /dev/null 2>&1
    then
        USER_HOME="/"
        return
    fi
    # grep -v “#” 去除注释情况 '# HOME=/home' 和 'HOME=/home' 共存情况
    local default_home=$(grep -E "HOME=" "${useradd_file}" | grep -v "#" | awk -F"=" '{print $2}')
    if [ -z "${default_home}" ]; then
        USER_HOME="/home"
    else
        USER_HOME="${default_home}"
    fi
}
get_home_path


# shell read设置超时时间300s
READ_TIMEOUT=300
DEVKIT_USER="devkit"
DEVKIT_HOME="${USER_HOME}/${DEVKIT_USER}/"
CONFIG_FILE="${DEVKIT_HOME}/devkit_conf/install.ini"
SUCCESS=0
ERROR=1
DOING=2
ECHO=3
# 插件安装所需空间1G（server和agent相等）
INSTALL_SPACE_NEED=1000000
# 工具使用空间阈值10G
TOOL_USE_SPACE=10000000
DEVKIT_SUDO_TOOLS="/usr/bin/devkit_tools"
IPERF_PACKAGE="iperf3.tar.gz"
NET_FLOW_PACKAGE="Sys-Diagnosis-trace_net_flow.zip"
IPERF="iperf"
SUDO_FILE="/etc/sudoers.d/devkit_sys_diagnosis_sudoers"
SUDO_FILES_LIST=("hyper_tuner_memleak" "bcc_wrapper.sh" "check_user_password_diagnose" "leak_check.sh" "clean_memdiagnostic.sh" "tcpdump.sh" "iperf3.sh" "fio.sh" "netflow.sh")
OS_PASSWD="/etc/passwd"
OS_RELEASE="/etc/os-release"
OS_TYPE_LIST=('OpenEulerOS' 'EulerOS' 'CentOS' 'NeoKylinOS' 'SLESOS' 'UbuntuOS' 'DebianOS' 'LinxOS' 'KylinSecOS' 'KylinOS' 'UOS' 'iSoftOS' 'BCLinuxOS' 'UnionTechOS' 'ctyunOS')
# 框架OS映射插件OS（RedHatOS不支持ARM）
declare -A OS_MAP=(["DeepinOS"]="DebianOS" ["uosEulerOS"]="EulerOS")
declare -A PRE_LIB_LIST=(["OpenEulerOS"]="unzip" ["EulerOS"]="unzip" ["CentOS"]="unzip" ["NeoKylinOS"]="unzip" ["SLESOS"]="unzip" ["UbuntuOS"]="unzip" ["DebianOS"]="unzip" ["LinxOS"]="unzip" ["KylinSecOS"]="unzip" ["KylinOS"]="unzip" ["UOS"]="unzip" ["iSoftOS"]="unzip" ["BCLinuxOS"]="unzip" ["UnionTechOS"]="unzip" ["ctyunOS"]="unzip")
declare -A PRE_CMD_LIST=(["unzip"]="unzip -v")
PACKAGE_TOOLS_LIST=("yum" "apt" "zypper")
declare -A COMMAND_LIST=(["OpenEulerOS"]="yum" ["EulerOS"]="yum" ["CentOS"]="yum" ["NeoKylinOS"]="yum" ["SLESOS"]="zypper" ["UbuntuOS"]="apt" ["DebianOS"]="apt" ["LinxOS"]="apt" ["KylinSecOS"]="yum" ["KylinOS"]="yum" ["UOS"]="apt" ["iSoftOS"]="yum"  ["UnionTechOS"]="yum" ["BCLinuxOS"]="yum" ["ctyunOS"]="yum")
declare -A ESSENTIAL_LIB_LIST=(["OpenEulerOS"]="make expect perf gcc-c++ gcc glibc openssl sudo util-linux binutils dmidecode sysstat numactl sqlite perl logrotate curl zip bash tar rm which gzip cat ls df grep awk rpm \
                               libffi-devel pcre pcre-devel zlib zlib-devel libunwind openssl-devel"  # OpenEulerOS
                               ["EulerOS"]="make expect perf gcc-c++ gcc glibc openssl sudo util-linux binutils dmidecode sysstat numactl sqlite perl logrotate curl zip bash tar rm which gzip cat ls df grep awk rpm \
                               libffi-devel pcre pcre-devel zlib zlib-devel libunwind openssl-devel"  # EulerOS
                               ["CentOS"]="make expect perf gcc-c++ gcc glibc openssl sudo util-linux binutils dmidecode sysstat numactl sqlite perl logrotate curl zip bash tar rm which gzip cat ls df grep awk rpm \
                               libffi-devel pcre pcre-devel zlib zlib-devel libunwind openssl-devel"  # CentOS
                               ["NeoKylinOS"]="make expect perf gcc-c++ gcc glibc openssl sudo util-linux binutils dmidecode sysstat numactl sqlite perl logrotate curl zip bash tar rm which gzip cat ls df grep awk rpm \
                               libffi-devel pcre pcre-devel zlib zlib-devel libunwind openssl-devel"  # NeoKylinOS
                               ["SLESOS"]="make expect perf gcc-c++ gcc glibc openssl sudo util-linux binutils dmidecode sysstat numactl sqlite3 perl logrotate curl zip bash tar rm which gzip cat ls df grep awk\
                               libffi-devel pcre-tools pcre-devel pcre2-devel libz1 zlib-devel libunwind libopenssl-devel"  # SLESOS
                               ["UbuntuOS"]="make expect linux-tools-common g++ gcc libc-bin openssl sudo util-linux binutils dmidecode sysstat numactl sqlite3 perl logrotate curl zip bash tar rm which gzip cat ls df grep awk\
                               linux-tools-version linux-tools-generic tcl-expect libffi-dev libpcre16-3 libpcre3-dev libpcre32-3 libpcre3 libpcrecpp0v5 zlib1g zlib1g-dev libunwind8 libssl-dev"  # UbuntuOS
                               ["DebianOS"]="make expect linux-perf-4.19 g++ gcc libc-bin openssl sudo util-linux binutils dmidecode sysstat numactl sqlite3 perl logrotate curl zip bash tar rm which gzip cat ls df grep awk\
                               linux-perf tcl-expect libffi-dev libpcre16-3 libpcre3-dev libpcre32-3 libpcre3 libpcrecpp0v5 zlib1g zlib1g-dev libunwind8 libssl-dev"  # DebianOS
                               ["LinxOS"]="make expect linux-perf-4.19 g++ gcc libc-bin openssl sudo util-linux binutils dmidecode sysstat numactl sqlite3 perl logrotate curl zip bash tar rm which gzip cat ls df grep awk\
                               linux-perf tcl-expect libffi-dev libpcre16-3 libpcre3-dev libpcre32-3 libpcre3 libpcrecpp0v5 zlib1g zlib1g-dev libunwind8 libssl-dev"  # LinxOS
                               ["KylinSecOS"]="make expect perf gcc-c++ gcc glibc openssl sudo util-linux binutils dmidecode sysstat numactl sqlite perl logrotate curl zip bash tar rm which gzip cat ls df grep awk rpm \
                               libffi-devel pcre pcre-devel zlib zlib-devel libunwind openssl-devel"  # KylinSecOS
                               ["KylinOS"]="make expect perf gcc-c++ gcc glibc openssl sudo util-linux binutils dmidecode sysstat numactl sqlite perl logrotate curl zip bash tar rm which gzip cat ls df grep awk rpm \
                               libffi-devel pcre pcre-devel zlib zlib-devel libunwind openssl-devel"  # KylinOS
                               ["UOS"]="make expect linux-perf-4.19 g++ gcc libc-bin openssl sudo util-linux binutils dmidecode sysstat numactl sqlite3 perl logrotate curl zip bash tar rm which gzip cat ls df grep awk \
                               linux-perf tcl-expect libffi-dev libpcre16-3 libpcre3-dev libpcre32-3 libpcre3 libpcrecpp0v5 zlib1g zlib1g-dev libunwind8 libssl-dev"  # UOS
                               ["iSoftOS"]="make expect perf gcc-c++ gcc glibc openssl sudo util-linux binutils dmidecode sysstat numactl sqlite perl logrotate curl zip bash tar rm which gzip cat ls df grep awk rpm \
                               libffi-devel pcre pcre-devel zlib zlib-devel libunwind openssl-devel"  # iSoftOS
                               ["UnionTechOS"]="make expect perf gcc-c++ gcc glibc openssl sudo util-linux binutils dmidecode sysstat numactl sqlite perl logrotate curl zip bash tar rm which gzip cat ls df grep awk rpm \
                               libffi-devel pcre pcre-devel zlib zlib-devel libunwind openssl-devel"  # UnionTechOS
                               ["BCLinuxOS"]="make expect perf gcc-c++ gcc glibc openssl sudo util-linux binutils dmidecode sysstat numactl sqlite perl logrotate curl zip bash tar rm which gzip cat ls df grep awk rpm \
                               libffi-devel pcre pcre-devel zlib zlib-devel libunwind openssl-devel"  # BCLinuxOS
                               ["ctyunOS"]="make expect perf gcc-c++ gcc glibc openssl sudo util-linux binutils dmidecode sysstat numactl sqlite perl logrotate curl zip bash tar rm which gzip cat ls df grep awk rpm \
                               libffi-devel pcre pcre-devel zlib zlib-devel libunwind openssl-devel"  # ctyunOS
)

declare -A ESSENTIAL_CMD_LIST=(["make"]="make -v" ["expect"]="expect -v" ["perf"]="perf -v" ["linux-tools-common"]="perf -v" ["linux-perf-4.19"]="perf -v" ["gcc-c++"]="g++ -v" ["g++"]="g++ -v" ["gcc"]="gcc -v" ["glibc"]="ldd --version" ["libc-bin"]="ldd --version" ["sudo"]="sudo -V" ["util-linux"]="lscpu -V" ["binutils"]="addr2line -v" ["dmidecode"]="dmidecode -V" \
                    ["sysstat"]="pidstat -V" ["numactl"]="numactl -H" ["sqlite"]="sqlite3 --version" ["sqlite3"]="sqlite3 --version" ["perl"]="perl -V" ["logrotate"]="logrotate --version" ["curl"]="curl -V" ["zip"]="zip -v" ["bash"]="bash --version" \
                    ["tar"]="tar --version" ["rm"]="rm --version" ["which"]="type which" ["gzip"]="gzip -V" ["cat"]="cat --version" ["ls"]="ls --version" ["df"]="df --version" ["grep"]="grep -V" ["awk"]="type awk" ["rpm"]="rpm --version" \
                    ["libffi-devel"]="package_type" ["libffi-dev"]="package_type" ["pcre"]="package_type" ["pcre-devel"]="package_type" ["zlib"]="package_type" ["zlib-devel"]="package_type" ["libunwind"]="package_type" ["openssl-devel"]="package_type" \
                    ["pcre-tools"]="package_type" ["pcre2-devel"]="package_type" ["libz1"]="package_type" ["libopenssl-devel"]="package_type" ["linux-tools-version"]="package_type" ["linux-tools-generic"]="package_type" ["tcl-expect"]="package_type" \
                    ["libpcre16-3"]="package_type" ["libpcre3-dev"]="package_type" ["libpcre32-3"]="package_type" ["libpcre3"]="package_type" ["libpcrecpp0v5"]="package_type" ["zlib1g"]="package_type" ["zlib1g-dev"]="package_type" \
                    ["libunwind8"]="package_type" ["libssl-dev"]="package_type" ["linux-perf"]="package_type")

declare -A LIB_CHECK_LIST=(["pcre"]="libpcre.so" ["zlib"]="libz.so" ["libunwind"]="libunwind*.so*" ["openssl-devel"]="libssl*.so*" ["pcre-tools"]="libpcre.so" ["pcre2-devel"]="libpcre2*.so" ["libz1"]="libz.so" ["libopenssl-devel"]="libssl*.so*"
                           ["tcl-expect"]="libtcl*.so*" ["libpcre16-3"]="libpcre16.so*" ["libpcre32-3"]="libpcre32.so*" ["libpcrecpp0v5"]="libpcrecpp*.so*" ["zlib1g"]="lmzlibw.so" ["libunwind8"]="libunwind.so*" ["libssl-dev"]="libssl.so*")

# 更新规则：可使用cmd检验的，新增到crontabs之前，NON_ESSENTIAL_CMD_LIST里面新增对应的cmd；使用package管理工具检验的，新增到最后即可
declare -A NON_ESSENTIAL_LIB_LIST=(["OpenEulerOS"]="graphviz psmisc strace pciutils lsscsi procps initscripts policycoreutils ethtool smartmontools kmod net-tools rsyslog iputils traceroute tcpdump fio ipmitool man bc route arp ibv crontabs libaio-devel numactl-devel " # OpenEulerOS
                                   ["EulerOS"]="graphviz psmisc strace pciutils lsscsi procps initscripts policycoreutils ethtool smartmontools kmod net-tools rsyslog iputils traceroute tcpdump fio ipmitool man bc route arp ibv crontabs libaio-devel numactl-devel" # EulerOS
                                   ["CentOS"]="graphviz psmisc strace pciutils lsscsi procps initscripts policycoreutils ethtool smartmontools kmod net-tools rsyslog iputils traceroute tcpdump fio ipmitool man bc route arp ibv crontabs libaio-devel numactl-devel" # CentOS
                                   ["NeoKylinOS"]="graphviz psmisc strace pciutils lsscsi procps initscripts policycoreutils ethtool smartmontools kmod net-tools rsyslog iputils traceroute tcpdump fio ipmitool man bc route arp ibv crontabs libaio-devel numactl-devel" # NeoKylinOS
                                   ["SLESOS"]="graphviz psmisc strace pciutils lsscsi procps aaa_base policycoreutils ethtool smartmontools kmod net-tools-deprecated rsyslog iputils traceroute tcpdump fio ipmitool man bc route arp ibv cron libaio-devel kmod-compat net-tools libnuma-devel" # SLESOS
                                   ["UbuntuOS"]="graphviz psmisc strace pciutils lsscsi procps sysvinit-utils policycoreutils ethtool smartmontools kmod net-tools rsyslog iputils-ping traceroute tcpdump fio ipmitool man bc route arp ibv cron ufw libaio-dev libnuma-dev" # UbuntuOS
                                   ["DebianOS"]="graphviz psmisc strace pciutils lsscsi procps sysvinit-utils policycoreutils ethtool smartmontools kmod net-tools rsyslog iputils-ping traceroute tcpdump fio ipmitool man bc route arp ibv cron ufw libaio-dev libnuma-dev" # DebianOS
                                   ["LinxOS"]="graphviz psmisc strace pciutils lsscsi procps sysvinit-utils policycoreutils ethtool smartmontools kmod net-tools rsyslog iputils-ping traceroute tcpdump fio ipmitool man bc route arp ibv cron ufw libaio-dev libnuma-dev" #  LinxOS（graphviz缺失）
                                   ["KylinSecOS"]="graphviz psmisc strace pciutils lsscsi procps initscripts policycoreutils ethtool smartmontools kmod net-tools rsyslog iputils traceroute tcpdump fio ipmitool man bc route arp ibv crontabs libaio-devel numactl-devel" # KylinSecOS
                                   ["KylinOS"]="graphviz psmisc strace pciutils lsscsi procps initscripts policycoreutils ethtool smartmontools kmod net-tools rsyslog iputils traceroute tcpdump fio ipmitool man bc route arp ibv crontabs libaio-devel numactl-devel" # KylinOS
                                   ["UOS"]="graphviz psmisc strace pciutils lsscsi procps sysvinit-utils policycoreutils ethtool smartmontools kmod net-tools rsyslog iputils-ping traceroute tcpdump fio ipmitool man bc route arp ibv cron ufw libaio-dev libnuma-dev" # UOS
                                   ["iSoftOS"]="graphviz psmisc strace pciutils lsscsi procps initscripts policycoreutils ethtool smartmontools kmod net-tools rsyslog iputils traceroute tcpdump fio ipmitool man bc route arp ibv crontabs libaio-devel numactl-devel" # iSoftOS
                                   ["UnionTechOS"]="graphviz psmisc strace pciutils lsscsi procps initscripts policycoreutils ethtool smartmontools kmod net-tools rsyslog iputils traceroute tcpdump fio ipmitool man bc route arp ibv crontabs libaio-devel numactl-devel" # UnionTechOS
                                   ["BCLinuxOS"]="graphviz psmisc strace pciutils lsscsi procps initscripts policycoreutils ethtool smartmontools kmod net-tools rsyslog iputils traceroute tcpdump fio ipmitool man bc route arp ibv crontabs libaio-devel numactl-devel" # BCLinuxOS
                                    ["ctyunOS"]="graphviz psmisc strace pciutils lsscsi procps initscripts policycoreutils ethtool smartmontools kmod net-tools rsyslog iputils traceroute tcpdump fio ipmitool man bc route arp ibv crontabs libaio-devel numactl-devel " # ctyunOS
)

declare -A NON_ESSENTIAL_CMD_LIST=(["graphviz"]="dot -V" ["psmisc"]="pstree -V" ["strace"]="strace -V" ["pciutils"]="lspci -v" ["lsscsi"]="lsscsi -V" ["procps"]="sysctl -V" ["initscripts"]="service -h" ["sysvinit-utils"]="service -h" ["policycoreutils"]="sestatus -v" ["ethtool"]="ethtool --version" ["smartmontools"]="smartctl -V" \
                        ["kmod"]="kmod -V" ["net-tools"]="netstat -V" ["rsyslog"]="rsyslogd -version" ["iputils"]="ping -V" ["iputils-ping"]="ping -V" ["traceroute"]="traceroute -V" ["tcpdump"]="tcpdump --version" ["fio"]="fio -v" ["ipmitool"]="ipmitool -V" ["man"]="man -V" ["bc"]="bc -v" \
                        ["route"]="route -n" ["arp"]="type arp" ["ibv"]="ibv_devinfo" ["crontabs"]="package_type" ["libaio-devel"]="package_type" ["libaio-dev"]="package_type" ["libnuma-dev"]="package_type" ["numactl-devel"]="package_type" ["cron"]="package_type" ["kmod-compat"]="package_type" ["libnuma-devel"]="package_type" ["ufw"]="package_type")

declare -A AGENT_SKIP_LIST=(["OpenEulerOS"]="expect graphviz sqlite pcre logrotate"
                            ["EulerOS"]="expect graphviz sqlite pcre logrotate"
                            ["CentOS"]="expect graphviz sqlite pcre logrotate"
                            ["NeoKylinOS"]="expect graphviz sqlite pcre logrotate"
                            ["SLESOS"]="expect graphviz sqlite3 pcre-tools logrotate"
                            ["UbuntuOS"]="expect graphviz sqlite3 logrotate"
                            ["DebianOS"]="expect graphviz sqlite3 logrotate"
                            ["LinxOS"]="expect graphviz sqlite3 logrotate"
                            ["KylinSecOS"]="expect graphviz sqlite pcre logrotate"
                            ["KylinOS"]="expect graphviz sqlite pcre logrotate"
                            ["UOS"]="expect graphviz sqlite3 logrotate"
                            ["iSoftOS"]="expect graphviz sqlite pcre logrotate"
                            ["UnionTechOS"]="expect graphviz sqlite pcre logrotate"
                            ["BCLinuxOS"]="expect graphviz sqlite pcre logrotate"
                            ["ctyunOS"]="expect graphviz sqlite pcre logrotate"
)

architecture=$(uname -m)
KUNPENG920_ID="0x00000000481fd010"
KUNPENG920B_ID="0x00000000480fd020"
KUNPENG920C_ID="0x00000000480fd030"
KUNPENG920F_ID="0x00000000480fd220"
CPU_ID_FILE="/sys/devices/system/cpu/cpu0/regs/identification/midr_el1"
# 支持的ARM平台CPUID集合
SUPPORTED_ARM=("${KUNPENG920_ID}" "${KUNPENG920B_ID}" "${KUNPENG920C_ID}" "${KUNPENG920F_ID}")
