#!/bin/bash
#Description: 框架安装插件时调用
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2023. All rights reserved.
#History: 2023-11-04 created

current_dir=$(cd $(dirname $0); pwd)
source "${current_dir}/utils.sh"
INSTALL_PATH="$1" # /opt/DevKit
PLUGIN_FLAG="$2" # web


function init_variable() {
    if [[ ! -f "${CONFIG_FILE}" ]]; then
        echo_and_log "There is no configure file(${CONFIG_FILE}), please check!" "${ERROR}"
        exit "${ERROR}"
    fi
    os_type=$(grep -w OS_TYPE "${CONFIG_FILE}" | awk -F= '{if($2 !="") print $2}')
    INSTALL_LOG_PATH="${INSTALL_PATH}/logs/sys_diagnosis_install.log"
    INSTALL_PLUGIN_PATH="${INSTALL_PATH}/devkitplugins/"
    PYTHON_PATH="${INSTALL_PATH}/tools/python_venv/bin/python3"
}

function create_check_pd_result() {
    # 创建 check_pd_result 文件 用于采集
    local check_pd_result_path="${INSTALL_PATH}/tools/rpc_cluster/check_pd_result"
    if [[ ! -f "${check_pd_result_path}" ]]; then
        touch "${check_pd_result_path}"
        chown "${DEVKIT_USER}":"${DEVKIT_USER}" "${check_pd_result_path}"
        chmod 600 "${check_pd_result_path}"
    fi
}

function change_file_path_permission() {
    local diagnosis_path="${INSTALL_PLUGIN_PATH}/sys_diagnosis/"
    if [[ -d "${diagnosis_path}" ]]; then
        # 去除符合条件的文件的other权限
        find "${diagnosis_path}" -user "${DEVKIT_USER}" -type f -perm /007 -exec chmod o= {} \;
        find "${diagnosis_path}" -type d -exec chmod 700 {} \;
        chmod 755 "${diagnosis_path}"
        chown root:root "${diagnosis_path}"
    fi
    find "${diagnosis_path}/deploy" -type f -name "*\.sh" -exec chown root:root {} \; >> /dev/null 2>&1
    # 所有so权限更改为550
    find "${diagnosis_path}" -type f -name "*\.so" -print0 | xargs -0 chmod 550 >> /dev/null 2>&1
    find "${diagnosis_path}" -type f -name "*\.so\.*" -print0 | xargs -0 chmod 550 >> /dev/null 2>&1

    find "${diagnosis_path}" -type f -name "*\.ini" -print0 | xargs -0 chmod 600 >> /dev/null 2>&1
    find "${diagnosis_path}" -type f -name "*\.json" -print0 | xargs -0 chmod 400 >> /dev/null 2>&1
    find "${diagnosis_path}" -type f -name "Open_Source_Software_Notice\.txt" -print0 | xargs -0 chmod 400 >> /dev/null 2>&1
}

function load_data() {
    export LD_LIBRARY_PATH=${DEVKIT_HOME}/libsqlite3:${DEVKIT_HOME}/libssl:${DEVKIT_HOME}/lib
    source "${INSTALL_PATH}/tools/python_venv/bin/activate"
    python3 "${INSTALL_PLUGIN_PATH}/manage.pyc" loaddata "${INSTALL_PLUGIN_PATH}/sys_diagnosis/system_config.json" >> /dev/null 2>&1
    python3 "${INSTALL_PLUGIN_PATH}/manage.pyc" loaddata "${INSTALL_PLUGIN_PATH}/sys_diagnosis/storage.json" >> /dev/null 2>&1
    rm -f "${INSTALL_PLUGIN_PATH}/sys_diagnosis/system_config.json"
    rm -f "${INSTALL_PLUGIN_PATH}/sys_diagnosis/storage.json"
    deactivate
    echo_and_log "Load database success" "${SUCCESS}" "${INSTALL_LOG_PATH}"
}

function create_export_import_dir() {
    local export_dir="${INSTALL_PATH}/tools/rpc_cluster/server/exports/"
    local import_dir="${INSTALL_PATH}/tools/rpc_cluster/server/imports/"
    local django_dir="${INSTALL_PATH}/tools/rpc_cluster/server/django/"
    if [[ ! -d "${export_dir}" ]]; then
         mkdir "${export_dir}"
         chown "${DEVKIT_USER}":"${DEVKIT_USER}" "${export_dir}"
         chmod 700 "${export_dir}"
    fi
    if [[ ! -d "${import_dir}" ]]; then
         mkdir "${import_dir}"
         chown "${DEVKIT_USER}":"${DEVKIT_USER}" "${import_dir}"
         chmod 700 "${import_dir}"
    fi
    if [[ ! -d "${django_dir}" ]]; then
         mkdir "${django_dir}"
         mkdir "${django_dir}/exports"
         mkdir "${django_dir}/imports"
         chown -R "${DEVKIT_USER}":"${DEVKIT_USER}" "${django_dir}"
         chmod -R 700 "${django_dir}"
    fi
}

function main() {
    init_variable
    create_log "${INSTALL_LOG_PATH}"
    echo_and_log "Run sys_diagnosis install script" "${DOING}" "${INSTALL_LOG_PATH}"
    check_kunpeng "${INSTALL_LOG_PATH}"
    if [[ "${PLUGIN_FLAG}" == "web" ]]; then
        # 强制安装缺失依赖
        bash "${current_dir}/env_check.sh" "${INSTALL_PATH}" "${PLUGIN_FLAG}"
        local check_res="$?"
        export LD_LIBRARY_PATH=${DEVKIT_HOME}/libsqlite3:${DEVKIT_HOME}/libssl:${DEVKIT_HOME}/lib
        if [[ -f "${current_dir}/make_env_check_result.py" ]]; then
            ${PYTHON_PATH} "${current_dir}/make_env_check_result.py"
        else
            ${PYTHON_PATH} "${current_dir}/make_env_check_result.pyc"
        fi
        unset LD_LIBRARY_PATH
        local temp_env_json="${current_dir}/sys_diagnosis_env_res.json"
        local final_env_json_dir="${INSTALL_PATH}/devkitplugins/check_env_result"
        local final_env_json="${final_env_json_dir}/sys_diagnosis_env_res.json"
        if [[ -f "${temp_env_json}" ]] && [[ -d "${final_env_json_dir}" ]]; then
            mv "${temp_env_json}" "${final_env_json}"
            chmod 600 "${final_env_json}"
            chown "${DEVKIT_USER}":"${DEVKIT_USER}" "${final_env_json}"
        fi
        if [[ "${check_res}" -ne "${SUCCESS}" ]]; then
            echo_and_log "There are dependencies that do not meet the installation requirements" "${ERROR}" "${INSTALL_LOG_PATH}"
            exit "${ERROR}"
        else
            echo_and_log "The environment check is passed" "${SUCCESS}" "${INSTALL_LOG_PATH}"
        fi
    else
        # 磁盘空间大小检测
        echo_and_log "Check disk space" "${DOING}" "${INSTALL_LOG_PATH}"
        disk_space_check "${INSTALL_PATH}" "${INSTALL_SPACE_NEED}"
        if [[ "$?" -ne "${SUCCESS}" ]]; then
            echo_and_log "Check install path disk space Failed" "${ERROR}" "${INSTALL_LOG_PATH}"
            exit "${ERROR}"
        fi
        echo_and_log "Check install path disk space Success" "${SUCCESS}" "${INSTALL_LOG_PATH}"
    fi
    echo_and_log "Creating an Application Running User" "${DOING}" "${INSTALL_LOG_PATH}"
    create_launcher "${os_type}"
    echo_and_log "Creating an Application Running User" "${SUCCESS}" "${INSTALL_LOG_PATH}"
    copy_sudo_files "${INSTALL_PATH}" "${INSTALL_LOG_PATH}"
    add_sudoers "${INSTALL_PATH}" "${INSTALL_LOG_PATH}"
    install_third_app "${INSTALL_LOG_PATH}"
    cp "${IPERF_PACKAGE}" "${NET_FLOW_PACKAGE}" "${INSTALL_PLUGIN_PATH}/sys_diagnosis/deploy/"
    load_data
    create_export_import_dir
    create_check_pd_result
    change_file_path_permission
    echo_and_log "Run sys_diagnosis install script success" "${SUCCESS}" "${INSTALL_LOG_PATH}"
}


main $@
