#!/bin/bash
#Description: DevKit install tool
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2023. All rights reserved.
#History: 2023-5-25 created

current_dir=$(cd $(dirname $0); pwd)
source "${current_dir}/utils.sh"
INSTALL_PATH="$1"   # /opt/DevKit


function init_variable() {
    if [[ ! -f "${CONFIG_FILE}" ]]; then
        echo_and_log "There is no configure file(${CONFIG_FILE}), please check!" "${ERROR}"
        exit "${ERROR}"
    fi
    UNINSTALL_LOG_PATH="${INSTALL_PATH}/logs/sys_diagnosis_uninstall.log"
    if [ ! -e "${UNINSTALL_LOG_PATH}" ]; then
        touch "${UNINSTALL_LOG_PATH}"
        chown devkit:devkit "${UNINSTALL_LOG_PATH}"
        chmod 600 "${UNINSTALL_LOG_PATH}"
    fi
}

function delete_task_dir() {
    local task_ids
    task_ids=$(sqlite3 "${INSTALL_PATH}/devkitplugins/sqlite/db.sqlite3" "select task_id from diagnostic_taskmanager_task;")
    for task_id in ${task_ids[@]}; do
        # server任务目录
        if [[ -d "${INSTALL_PATH}/tools/rpc_cluster/server/storage/server/${task_id}" ]]; then
            rm -rf "${INSTALL_PATH}/tools/rpc_cluster/server/storage/server/${task_id}"
        fi
        # client任务目录
        if [[ -d "${INSTALL_PATH}/tools/rpc_cluster/client/storage/client/${task_id}" ]]; then
            rm -rf "${INSTALL_PATH}/tools/rpc_cluster/client/storage/client/${task_id}"
        fi
        # 导出任务目录
        rm -rf "${INSTALL_PATH}/tools/rpc_cluster/server/exports/${task_id}_"*
        if [[ -f "${INSTALL_PATH}/tools/rpc_cluster/server/django/exports/task-${task_id}.tar.gz" ]]; then
            rm -f "${INSTALL_PATH}/tools/rpc_cluster/server/django/exports/task-${task_id}.tar.gz"
        fi
        # 导入任务目录
        if [[ -f "${INSTALL_PATH}/tools/rpc_cluster/server/django/imports/task-${task_id}.tar.gz" ]]; then
            rm -f "${INSTALL_PATH}/tools/rpc_cluster/server/django/imports/task-${task_id}.tar.gz"
        fi
    done
    local import_failed_tasks
    import_failed_tasks=$(sqlite3 "${INSTALL_PATH}/devkitplugins/sqlite/db.sqlite3" "select file_path from diagnostic_importexporttasks_importexporttasktemplate where task_id=-1;")
    for import_failed_task_path in ${import_failed_tasks[@]}; do
        if [[ -d "${import_failed_task_path}" ]]; then
            local failed_task_tar
            failed_task_tar="${import_failed_task_path}/$(ls ${import_failed_task_path} | grep .tar)"
            if [[ -f "${failed_task_tar}" ]]; then
                # 格式 task-64b413f280ce4b10b554d658f4569829.tar.gz
                local failed_task_tar_id
                failed_task_tar_id=$(tar -tf "${failed_task_tar}" | grep task-)
                if [[ -f "${INSTALL_PATH}/tools/rpc_cluster/server/django/imports/${failed_task_tar_id}" ]]; then
                    rm -f "${INSTALL_PATH}/tools/rpc_cluster/server/django/imports/${failed_task_tar_id}"
                fi
                # 格式 64b413f280ce4b10b554d658f4569829
                local failed_task_id="${failed_task_tar_id:5:32}"
                if [[ -d "${INSTALL_PATH}/tools/rpc_cluster/server/storage/server/${failed_task_id}" ]]; then
                    rm -rf "${INSTALL_PATH}/tools/rpc_cluster/server/storage/server/${failed_task_id}"
                fi
            fi
        fi
    done
    # 导入任务目录（删除系统诊断的导入任务）
    if [[ -d "${INSTALL_PATH}/tools/rpc_cluster/server/imports" ]]; then
        find "${INSTALL_PATH}"/tools/rpc_cluster/server/imports/* -prune -name sys_diagnosis_* -exec rm -rf {} \; >> /dev/null 2>&1
    fi
    echo_and_log "Task data deleted successfully." "${SUCCESS}" "${UNINSTALL_LOG_PATH}"
}

# 删除冗余的表，未在APP中注册，框架无法删除，需要插件自行处理
function delete_redundant_tables() {
    local tables=("diagnostic_usermanager_failedlogin" "diagnostic_usermanager_user")
    for ((i=0;i<${#tables[@]};i++))
    do
        sqlite3 "${INSTALL_PATH}/devkitplugins/sqlite/db.sqlite3" "DROP TABLE IF EXISTS ${tables[$i]};"
    done
}

# 通过框架卸载插件时，已经删除了devkitplugins下的sys_diagnosis插件目录，此处只需要删除其余残留的文件
function remove_files() {
    delete_sudo_tools "${UNINSTALL_LOG_PATH}"

    local files_to_be_deleted=()
    for ((i=0;i<${#files_to_be_deleted[@]};i++))
    do
        local file_or_dir="${files_to_be_deleted[$i]}"
        if [[ -d "${file_or_dir}" ]]; then
            rm -rf "${file_or_dir}"
            echo_and_log "Delete ${file_or_dir} dir success" "${SUCCESS}" "${UNINSTALL_LOG_PATH}"
        elif [[ -f "${file_or_dir}" ]]; then
            rm -f "${file_or_dir}"
            echo_and_log "Delete ${file_or_dir} file success" "${SUCCESS}" "${UNINSTALL_LOG_PATH}"
        else
            echo_and_log "File ${file_or_dir} not exists, continue" "${DOING}" "${UNINSTALL_LOG_PATH}"
        fi
    done

    delete_task_dir
}

function main() {
    init_variable
    delete_redundant_tables
    remove_files
}

main $@
